@@ -1,7 +1,7 @@
 from __future__ import annotations
 from typing import TypeVar, Any, Callable
 from . import utils, io_safe, config
-from . session import DDBSession
+from . sessions import SessionFileFull, SessionFileKey, SessionFileWhere, SessionDirFull, SessionDirWhere
 
 T = TypeVar("T")
 
@@ -81,9 +81,13 @@ def at(*path, key: str = None, where: Callable[[Any, Any], bool] = None) -> DDBM
 
 
 class DDBMethodChooser:
+
+	__slots__ = ("path", "key", "where", "op_type")
+
 	path: str
 	key: str
 	where: Callable[[Any, Any], bool]
+	op_type: OperationType
 
 
 	def __init__(self, path: tuple, key: str = None, where: Callable[[Any, Any], bool] = None):
@@ -112,15 +116,12 @@ class DDBMethodChooser:
 		if self.where is not None:
 			raise RuntimeError("DDB.at(where=...).exists() cannot be used with the where parameter")
 
-		if len(utils.find(self.path)) == 0:
+		if len(utils.find_all(self.path)) == 0:
 			return False
 		if self.key is None:
 			return True
 		# Key is passed and occurs is True
-		try:
-			return io_safe.partial_read(self.path, key=self.key) is not None
-		except KeyError:
-			return False
+		return io_safe.partial_read(self.path, key=self.key) is not None
 
 
 	def create(self, data=None, force_overwrite: bool = False):
@@ -130,7 +131,7 @@ class DDBMethodChooser:
 		`force_overwrite` is set to True.
 
 		Args:
-		- `data`: The data to write to the file. If not specified, an empty dict
+		- `data`: The data to write to the file. If not specified, it will be `{}`
 		will be written.
 		- `force_overwrite`: If `True`, will overwrite the file if it already
 		exists, defaults to False (optional).
@@ -176,9 +177,6 @@ class DDBMethodChooser:
 			data = io_safe.read(self.path)
 
 		elif self.op_type.file_key:
-			_, json_exists, _, ddb_exists = utils.db_paths(self.path)
-			if not json_exists and not ddb_exists:
-				return None
 			data = io_safe.partial_read(self.path, self.key)
 
 		elif self.op_type.file_where:
@@ -190,11 +188,11 @@ class DDBMethodChooser:
 					data[k] = v
 
 		elif self.op_type.dir_normal:
-			pattern_paths = utils.find(self.path)
+			pattern_paths = utils.find_all(self.path)
 			data = {n.split("/")[-1]: io_safe.read(n) for n in pattern_paths}
 
 		elif self.op_type.dir_where:
-			for db_name in utils.find(self.path):
+			for db_name in utils.find_all(self.path):
 				k, v = db_name.split("/")[-1], io_safe.read(db_name)
 				if self.where(k, type_cast(v)):
 					data[k] = v
@@ -202,7 +200,7 @@ class DDBMethodChooser:
 		return type_cast(data)
 
 
-	def session(self, as_type: T = None) -> DDBSession[T]:
+	def session(self, as_type: T = None) -> SessionFileFull[T] | SessionFileKey[T] | SessionFileWhere[T] | SessionDirFull[T] | SessionDirWhere[T]:
 		"""
 		Opens a session to the selected file(s) or folder, depending on previous
 		`.at(...)` selection. Inside the with block, you have exclusive access
@@ -212,5 +210,18 @@ class DDBMethodChooser:
 		Args:
 		- `as_type`: If provided, cast the value to the given type.
 		Eg. as_type=str will return str(value).
+
+		Raises:
+		- `FileNotFoundError`: If the file does not exist.
+		- `KeyError`: If a key is specified and it does not exist.
 		"""
-		return DDBSession(self.path, self.op_type, self.key, self.where, as_type)
+		if self.op_type.file_normal:
+			return SessionFileFull(self.path, as_type)
+		if self.op_type.file_key:
+			return SessionFileKey(self.path, self.key, as_type)
+		if self.op_type.file_where:
+			return SessionFileWhere(self.path, self.where, as_type)
+		if self.op_type.dir_normal:
+			return SessionDirFull(self.path, as_type)
+		if self.op_type.dir_where:
+			return SessionDirWhere(self.path, self.where, as_type)
